# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import dataclasses
import json  # type: ignore
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import gapic_v1, operations_v1, rest_helpers, rest_streaming
from google.api_core import exceptions as core_exceptions
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.requests import AuthorizedSession  # type: ignore
from google.cloud.location import locations_pb2  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
from google.protobuf import json_format
from requests import __version__ as requests_version

from google.cloud.discoveryengine_v1alpha.types import engine
from google.cloud.discoveryengine_v1alpha.types import engine as gcd_engine
from google.cloud.discoveryengine_v1alpha.types import engine_service

from .base import DEFAULT_CLIENT_INFO as BASE_DEFAULT_CLIENT_INFO
from .rest_base import _BaseEngineServiceRestTransport

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object, None]  # type: ignore


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=BASE_DEFAULT_CLIENT_INFO.gapic_version,
    grpc_version=None,
    rest_version=f"requests@{requests_version}",
)


class EngineServiceRestInterceptor:
    """Interceptor for EngineService.

    Interceptors are used to manipulate requests, request metadata, and responses
    in arbitrary ways.
    Example use cases include:
    * Logging
    * Verifying requests according to service or custom semantics
    * Stripping extraneous information from responses

    These use cases and more can be enabled by injecting an
    instance of a custom subclass when constructing the EngineServiceRestTransport.

    .. code-block:: python
        class MyCustomEngineServiceInterceptor(EngineServiceRestInterceptor):
            def pre_create_engine(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_create_engine(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_delete_engine(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_delete_engine(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_get_engine(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_get_engine(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_list_engines(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_list_engines(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_pause_engine(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_pause_engine(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_resume_engine(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_resume_engine(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_tune_engine(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_tune_engine(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_update_engine(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_update_engine(self, response):
                logging.log(f"Received response: {response}")
                return response

        transport = EngineServiceRestTransport(interceptor=MyCustomEngineServiceInterceptor())
        client = EngineServiceClient(transport=transport)


    """

    def pre_create_engine(
        self,
        request: engine_service.CreateEngineRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[engine_service.CreateEngineRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for create_engine

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EngineService server.
        """
        return request, metadata

    def post_create_engine(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for create_engine

        Override in a subclass to manipulate the response
        after it is returned by the EngineService server but before
        it is returned to user code.
        """
        return response

    def pre_delete_engine(
        self,
        request: engine_service.DeleteEngineRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[engine_service.DeleteEngineRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for delete_engine

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EngineService server.
        """
        return request, metadata

    def post_delete_engine(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for delete_engine

        Override in a subclass to manipulate the response
        after it is returned by the EngineService server but before
        it is returned to user code.
        """
        return response

    def pre_get_engine(
        self,
        request: engine_service.GetEngineRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[engine_service.GetEngineRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for get_engine

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EngineService server.
        """
        return request, metadata

    def post_get_engine(self, response: engine.Engine) -> engine.Engine:
        """Post-rpc interceptor for get_engine

        Override in a subclass to manipulate the response
        after it is returned by the EngineService server but before
        it is returned to user code.
        """
        return response

    def pre_list_engines(
        self,
        request: engine_service.ListEnginesRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[engine_service.ListEnginesRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for list_engines

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EngineService server.
        """
        return request, metadata

    def post_list_engines(
        self, response: engine_service.ListEnginesResponse
    ) -> engine_service.ListEnginesResponse:
        """Post-rpc interceptor for list_engines

        Override in a subclass to manipulate the response
        after it is returned by the EngineService server but before
        it is returned to user code.
        """
        return response

    def pre_pause_engine(
        self,
        request: engine_service.PauseEngineRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[engine_service.PauseEngineRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for pause_engine

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EngineService server.
        """
        return request, metadata

    def post_pause_engine(self, response: engine.Engine) -> engine.Engine:
        """Post-rpc interceptor for pause_engine

        Override in a subclass to manipulate the response
        after it is returned by the EngineService server but before
        it is returned to user code.
        """
        return response

    def pre_resume_engine(
        self,
        request: engine_service.ResumeEngineRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[engine_service.ResumeEngineRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for resume_engine

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EngineService server.
        """
        return request, metadata

    def post_resume_engine(self, response: engine.Engine) -> engine.Engine:
        """Post-rpc interceptor for resume_engine

        Override in a subclass to manipulate the response
        after it is returned by the EngineService server but before
        it is returned to user code.
        """
        return response

    def pre_tune_engine(
        self,
        request: engine_service.TuneEngineRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[engine_service.TuneEngineRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for tune_engine

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EngineService server.
        """
        return request, metadata

    def post_tune_engine(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for tune_engine

        Override in a subclass to manipulate the response
        after it is returned by the EngineService server but before
        it is returned to user code.
        """
        return response

    def pre_update_engine(
        self,
        request: engine_service.UpdateEngineRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[engine_service.UpdateEngineRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for update_engine

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EngineService server.
        """
        return request, metadata

    def post_update_engine(self, response: gcd_engine.Engine) -> gcd_engine.Engine:
        """Post-rpc interceptor for update_engine

        Override in a subclass to manipulate the response
        after it is returned by the EngineService server but before
        it is returned to user code.
        """
        return response

    def pre_cancel_operation(
        self,
        request: operations_pb2.CancelOperationRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[operations_pb2.CancelOperationRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for cancel_operation

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EngineService server.
        """
        return request, metadata

    def post_cancel_operation(self, response: None) -> None:
        """Post-rpc interceptor for cancel_operation

        Override in a subclass to manipulate the response
        after it is returned by the EngineService server but before
        it is returned to user code.
        """
        return response

    def pre_get_operation(
        self,
        request: operations_pb2.GetOperationRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[operations_pb2.GetOperationRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for get_operation

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EngineService server.
        """
        return request, metadata

    def post_get_operation(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for get_operation

        Override in a subclass to manipulate the response
        after it is returned by the EngineService server but before
        it is returned to user code.
        """
        return response

    def pre_list_operations(
        self,
        request: operations_pb2.ListOperationsRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[operations_pb2.ListOperationsRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for list_operations

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EngineService server.
        """
        return request, metadata

    def post_list_operations(
        self, response: operations_pb2.ListOperationsResponse
    ) -> operations_pb2.ListOperationsResponse:
        """Post-rpc interceptor for list_operations

        Override in a subclass to manipulate the response
        after it is returned by the EngineService server but before
        it is returned to user code.
        """
        return response


@dataclasses.dataclass
class EngineServiceRestStub:
    _session: AuthorizedSession
    _host: str
    _interceptor: EngineServiceRestInterceptor


class EngineServiceRestTransport(_BaseEngineServiceRestTransport):
    """REST backend synchronous transport for EngineService.

    Service for managing
    [Engine][google.cloud.discoveryengine.v1alpha.Engine] configuration.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends JSON representations of protocol buffers over HTTP/1.1
    """

    def __init__(
        self,
        *,
        host: str = "discoveryengine.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        url_scheme: str = "https",
        interceptor: Optional[EngineServiceRestInterceptor] = None,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'discoveryengine.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.

            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Callable[[], Tuple[bytes, bytes]]): Client
                certificate to configure mutual TLS HTTP channel. It is ignored
                if ``channel`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you are developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
            url_scheme: the protocol scheme for the API endpoint.  Normally
                "https", but for testing or local servers,
                "http" can be specified.
        """
        # Run the base constructor
        # TODO(yon-mg): resolve other ctor params i.e. scopes, quota, etc.
        # TODO: When custom host (api_endpoint) is set, `scopes` must *also* be set on the
        # credentials object
        super().__init__(
            host=host,
            credentials=credentials,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            url_scheme=url_scheme,
            api_audience=api_audience,
        )
        self._session = AuthorizedSession(
            self._credentials, default_host=self.DEFAULT_HOST
        )
        self._operations_client: Optional[operations_v1.AbstractOperationsClient] = None
        if client_cert_source_for_mtls:
            self._session.configure_mtls_channel(client_cert_source_for_mtls)
        self._interceptor = interceptor or EngineServiceRestInterceptor()
        self._prep_wrapped_messages(client_info)

    @property
    def operations_client(self) -> operations_v1.AbstractOperationsClient:
        """Create the client designed to process long-running operations.

        This property caches on the instance; repeated calls return the same
        client.
        """
        # Only create a new client if we do not already have one.
        if self._operations_client is None:
            http_options: Dict[str, List[Dict[str, str]]] = {
                "google.longrunning.Operations.CancelOperation": [
                    {
                        "method": "post",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*/branches/*/operations/*}:cancel",
                        "body": "*",
                    },
                    {
                        "method": "post",
                        "uri": "/v1alpha/{name=projects/*/locations/*/dataStores/*/branches/*/operations/*}:cancel",
                        "body": "*",
                    },
                ],
                "google.longrunning.Operations.GetOperation": [
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataConnector/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*/branches/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*/models/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*/schemas/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*/siteSearchEngine/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*/siteSearchEngine/targetSites/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/engines/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/dataStores/*/branches/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/dataStores/*/models/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/dataStores/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/evaluations/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/identity_mapping_stores/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/sampleQuerySets/*/operations/*}",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/operations/*}",
                    },
                ],
                "google.longrunning.Operations.ListOperations": [
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataConnector}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*/branches/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*/models/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*/schemas/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*/siteSearchEngine/targetSites}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*/siteSearchEngine}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/dataStores/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*/engines/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/collections/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/dataStores/*/branches/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/dataStores/*/models/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/dataStores/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*/identity_mapping_stores/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*/locations/*}/operations",
                    },
                    {
                        "method": "get",
                        "uri": "/v1alpha/{name=projects/*}/operations",
                    },
                ],
            }

            rest_transport = operations_v1.OperationsRestTransport(
                host=self._host,
                # use the credentials which are saved
                credentials=self._credentials,
                scopes=self._scopes,
                http_options=http_options,
                path_prefix="v1alpha",
            )

            self._operations_client = operations_v1.AbstractOperationsClient(
                transport=rest_transport
            )

        # Return the client from cache.
        return self._operations_client

    class _CreateEngine(
        _BaseEngineServiceRestTransport._BaseCreateEngine, EngineServiceRestStub
    ):
        def __hash__(self):
            return hash("EngineServiceRestTransport.CreateEngine")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )
            return response

        def __call__(
            self,
            request: engine_service.CreateEngineRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the create engine method over HTTP.

            Args:
                request (~.engine_service.CreateEngineRequest):
                    The request object. Request for
                [EngineService.CreateEngine][google.cloud.discoveryengine.v1alpha.EngineService.CreateEngine]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options = (
                _BaseEngineServiceRestTransport._BaseCreateEngine._get_http_options()
            )
            request, metadata = self._interceptor.pre_create_engine(request, metadata)
            transcoded_request = _BaseEngineServiceRestTransport._BaseCreateEngine._get_transcoded_request(
                http_options, request
            )

            body = _BaseEngineServiceRestTransport._BaseCreateEngine._get_request_body_json(
                transcoded_request
            )

            # Jsonify the query params
            query_params = _BaseEngineServiceRestTransport._BaseCreateEngine._get_query_params_json(
                transcoded_request
            )

            # Send the request
            response = EngineServiceRestTransport._CreateEngine._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
                body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_create_engine(resp)
            return resp

    class _DeleteEngine(
        _BaseEngineServiceRestTransport._BaseDeleteEngine, EngineServiceRestStub
    ):
        def __hash__(self):
            return hash("EngineServiceRestTransport.DeleteEngine")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: engine_service.DeleteEngineRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the delete engine method over HTTP.

            Args:
                request (~.engine_service.DeleteEngineRequest):
                    The request object. Request message for
                [EngineService.DeleteEngine][google.cloud.discoveryengine.v1alpha.EngineService.DeleteEngine]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options = (
                _BaseEngineServiceRestTransport._BaseDeleteEngine._get_http_options()
            )
            request, metadata = self._interceptor.pre_delete_engine(request, metadata)
            transcoded_request = _BaseEngineServiceRestTransport._BaseDeleteEngine._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEngineServiceRestTransport._BaseDeleteEngine._get_query_params_json(
                transcoded_request
            )

            # Send the request
            response = EngineServiceRestTransport._DeleteEngine._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_delete_engine(resp)
            return resp

    class _GetEngine(
        _BaseEngineServiceRestTransport._BaseGetEngine, EngineServiceRestStub
    ):
        def __hash__(self):
            return hash("EngineServiceRestTransport.GetEngine")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: engine_service.GetEngineRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> engine.Engine:
            r"""Call the get engine method over HTTP.

            Args:
                request (~.engine_service.GetEngineRequest):
                    The request object. Request message for
                [EngineService.GetEngine][google.cloud.discoveryengine.v1alpha.EngineService.GetEngine]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.engine.Engine:
                    Metadata that describes the training and serving
                parameters of an
                [Engine][google.cloud.discoveryengine.v1alpha.Engine].

            """

            http_options = (
                _BaseEngineServiceRestTransport._BaseGetEngine._get_http_options()
            )
            request, metadata = self._interceptor.pre_get_engine(request, metadata)
            transcoded_request = (
                _BaseEngineServiceRestTransport._BaseGetEngine._get_transcoded_request(
                    http_options, request
                )
            )

            # Jsonify the query params
            query_params = (
                _BaseEngineServiceRestTransport._BaseGetEngine._get_query_params_json(
                    transcoded_request
                )
            )

            # Send the request
            response = EngineServiceRestTransport._GetEngine._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = engine.Engine()
            pb_resp = engine.Engine.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_get_engine(resp)
            return resp

    class _ListEngines(
        _BaseEngineServiceRestTransport._BaseListEngines, EngineServiceRestStub
    ):
        def __hash__(self):
            return hash("EngineServiceRestTransport.ListEngines")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: engine_service.ListEnginesRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> engine_service.ListEnginesResponse:
            r"""Call the list engines method over HTTP.

            Args:
                request (~.engine_service.ListEnginesRequest):
                    The request object. Request message for
                [EngineService.ListEngines][google.cloud.discoveryengine.v1alpha.EngineService.ListEngines]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.engine_service.ListEnginesResponse:
                    Response message for
                [EngineService.ListEngines][google.cloud.discoveryengine.v1alpha.EngineService.ListEngines]
                method.

            """

            http_options = (
                _BaseEngineServiceRestTransport._BaseListEngines._get_http_options()
            )
            request, metadata = self._interceptor.pre_list_engines(request, metadata)
            transcoded_request = _BaseEngineServiceRestTransport._BaseListEngines._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = (
                _BaseEngineServiceRestTransport._BaseListEngines._get_query_params_json(
                    transcoded_request
                )
            )

            # Send the request
            response = EngineServiceRestTransport._ListEngines._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = engine_service.ListEnginesResponse()
            pb_resp = engine_service.ListEnginesResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_list_engines(resp)
            return resp

    class _PauseEngine(
        _BaseEngineServiceRestTransport._BasePauseEngine, EngineServiceRestStub
    ):
        def __hash__(self):
            return hash("EngineServiceRestTransport.PauseEngine")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )
            return response

        def __call__(
            self,
            request: engine_service.PauseEngineRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> engine.Engine:
            r"""Call the pause engine method over HTTP.

            Args:
                request (~.engine_service.PauseEngineRequest):
                    The request object. Request for pausing training of an
                engine.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.engine.Engine:
                    Metadata that describes the training and serving
                parameters of an
                [Engine][google.cloud.discoveryengine.v1alpha.Engine].

            """

            http_options = (
                _BaseEngineServiceRestTransport._BasePauseEngine._get_http_options()
            )
            request, metadata = self._interceptor.pre_pause_engine(request, metadata)
            transcoded_request = _BaseEngineServiceRestTransport._BasePauseEngine._get_transcoded_request(
                http_options, request
            )

            body = (
                _BaseEngineServiceRestTransport._BasePauseEngine._get_request_body_json(
                    transcoded_request
                )
            )

            # Jsonify the query params
            query_params = (
                _BaseEngineServiceRestTransport._BasePauseEngine._get_query_params_json(
                    transcoded_request
                )
            )

            # Send the request
            response = EngineServiceRestTransport._PauseEngine._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
                body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = engine.Engine()
            pb_resp = engine.Engine.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_pause_engine(resp)
            return resp

    class _ResumeEngine(
        _BaseEngineServiceRestTransport._BaseResumeEngine, EngineServiceRestStub
    ):
        def __hash__(self):
            return hash("EngineServiceRestTransport.ResumeEngine")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )
            return response

        def __call__(
            self,
            request: engine_service.ResumeEngineRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> engine.Engine:
            r"""Call the resume engine method over HTTP.

            Args:
                request (~.engine_service.ResumeEngineRequest):
                    The request object. Request for resuming training of an
                engine.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.engine.Engine:
                    Metadata that describes the training and serving
                parameters of an
                [Engine][google.cloud.discoveryengine.v1alpha.Engine].

            """

            http_options = (
                _BaseEngineServiceRestTransport._BaseResumeEngine._get_http_options()
            )
            request, metadata = self._interceptor.pre_resume_engine(request, metadata)
            transcoded_request = _BaseEngineServiceRestTransport._BaseResumeEngine._get_transcoded_request(
                http_options, request
            )

            body = _BaseEngineServiceRestTransport._BaseResumeEngine._get_request_body_json(
                transcoded_request
            )

            # Jsonify the query params
            query_params = _BaseEngineServiceRestTransport._BaseResumeEngine._get_query_params_json(
                transcoded_request
            )

            # Send the request
            response = EngineServiceRestTransport._ResumeEngine._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
                body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = engine.Engine()
            pb_resp = engine.Engine.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_resume_engine(resp)
            return resp

    class _TuneEngine(
        _BaseEngineServiceRestTransport._BaseTuneEngine, EngineServiceRestStub
    ):
        def __hash__(self):
            return hash("EngineServiceRestTransport.TuneEngine")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )
            return response

        def __call__(
            self,
            request: engine_service.TuneEngineRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the tune engine method over HTTP.

            Args:
                request (~.engine_service.TuneEngineRequest):
                    The request object. Request to manually start a tuning
                process now (instead of waiting for the
                periodically scheduled tuning to
                happen).
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.operations_pb2.Operation:
                    This resource represents a
                long-running operation that is the
                result of a network API call.

            """

            http_options = (
                _BaseEngineServiceRestTransport._BaseTuneEngine._get_http_options()
            )
            request, metadata = self._interceptor.pre_tune_engine(request, metadata)
            transcoded_request = (
                _BaseEngineServiceRestTransport._BaseTuneEngine._get_transcoded_request(
                    http_options, request
                )
            )

            body = (
                _BaseEngineServiceRestTransport._BaseTuneEngine._get_request_body_json(
                    transcoded_request
                )
            )

            # Jsonify the query params
            query_params = (
                _BaseEngineServiceRestTransport._BaseTuneEngine._get_query_params_json(
                    transcoded_request
                )
            )

            # Send the request
            response = EngineServiceRestTransport._TuneEngine._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
                body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = operations_pb2.Operation()
            json_format.Parse(response.content, resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_tune_engine(resp)
            return resp

    class _UpdateEngine(
        _BaseEngineServiceRestTransport._BaseUpdateEngine, EngineServiceRestStub
    ):
        def __hash__(self):
            return hash("EngineServiceRestTransport.UpdateEngine")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )
            return response

        def __call__(
            self,
            request: engine_service.UpdateEngineRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> gcd_engine.Engine:
            r"""Call the update engine method over HTTP.

            Args:
                request (~.engine_service.UpdateEngineRequest):
                    The request object. Request message for
                [EngineService.UpdateEngine][google.cloud.discoveryengine.v1alpha.EngineService.UpdateEngine]
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.gcd_engine.Engine:
                    Metadata that describes the training and serving
                parameters of an
                [Engine][google.cloud.discoveryengine.v1alpha.Engine].

            """

            http_options = (
                _BaseEngineServiceRestTransport._BaseUpdateEngine._get_http_options()
            )
            request, metadata = self._interceptor.pre_update_engine(request, metadata)
            transcoded_request = _BaseEngineServiceRestTransport._BaseUpdateEngine._get_transcoded_request(
                http_options, request
            )

            body = _BaseEngineServiceRestTransport._BaseUpdateEngine._get_request_body_json(
                transcoded_request
            )

            # Jsonify the query params
            query_params = _BaseEngineServiceRestTransport._BaseUpdateEngine._get_query_params_json(
                transcoded_request
            )

            # Send the request
            response = EngineServiceRestTransport._UpdateEngine._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
                body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = gcd_engine.Engine()
            pb_resp = gcd_engine.Engine.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_update_engine(resp)
            return resp

    @property
    def create_engine(
        self,
    ) -> Callable[[engine_service.CreateEngineRequest], operations_pb2.Operation]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._CreateEngine(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def delete_engine(
        self,
    ) -> Callable[[engine_service.DeleteEngineRequest], operations_pb2.Operation]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._DeleteEngine(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def get_engine(self) -> Callable[[engine_service.GetEngineRequest], engine.Engine]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._GetEngine(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def list_engines(
        self,
    ) -> Callable[
        [engine_service.ListEnginesRequest], engine_service.ListEnginesResponse
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ListEngines(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def pause_engine(
        self,
    ) -> Callable[[engine_service.PauseEngineRequest], engine.Engine]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._PauseEngine(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def resume_engine(
        self,
    ) -> Callable[[engine_service.ResumeEngineRequest], engine.Engine]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ResumeEngine(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def tune_engine(
        self,
    ) -> Callable[[engine_service.TuneEngineRequest], operations_pb2.Operation]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._TuneEngine(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def update_engine(
        self,
    ) -> Callable[[engine_service.UpdateEngineRequest], gcd_engine.Engine]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._UpdateEngine(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def cancel_operation(self):
        return self._CancelOperation(self._session, self._host, self._interceptor)  # type: ignore

    class _CancelOperation(
        _BaseEngineServiceRestTransport._BaseCancelOperation, EngineServiceRestStub
    ):
        def __hash__(self):
            return hash("EngineServiceRestTransport.CancelOperation")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )
            return response

        def __call__(
            self,
            request: operations_pb2.CancelOperationRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> None:
            r"""Call the cancel operation method over HTTP.

            Args:
                request (operations_pb2.CancelOperationRequest):
                    The request object for CancelOperation method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.
            """

            http_options = (
                _BaseEngineServiceRestTransport._BaseCancelOperation._get_http_options()
            )
            request, metadata = self._interceptor.pre_cancel_operation(
                request, metadata
            )
            transcoded_request = _BaseEngineServiceRestTransport._BaseCancelOperation._get_transcoded_request(
                http_options, request
            )

            body = _BaseEngineServiceRestTransport._BaseCancelOperation._get_request_body_json(
                transcoded_request
            )

            # Jsonify the query params
            query_params = _BaseEngineServiceRestTransport._BaseCancelOperation._get_query_params_json(
                transcoded_request
            )

            # Send the request
            response = EngineServiceRestTransport._CancelOperation._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
                body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            return self._interceptor.post_cancel_operation(None)

    @property
    def get_operation(self):
        return self._GetOperation(self._session, self._host, self._interceptor)  # type: ignore

    class _GetOperation(
        _BaseEngineServiceRestTransport._BaseGetOperation, EngineServiceRestStub
    ):
        def __hash__(self):
            return hash("EngineServiceRestTransport.GetOperation")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: operations_pb2.GetOperationRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the get operation method over HTTP.

            Args:
                request (operations_pb2.GetOperationRequest):
                    The request object for GetOperation method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                operations_pb2.Operation: Response from GetOperation method.
            """

            http_options = (
                _BaseEngineServiceRestTransport._BaseGetOperation._get_http_options()
            )
            request, metadata = self._interceptor.pre_get_operation(request, metadata)
            transcoded_request = _BaseEngineServiceRestTransport._BaseGetOperation._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEngineServiceRestTransport._BaseGetOperation._get_query_params_json(
                transcoded_request
            )

            # Send the request
            response = EngineServiceRestTransport._GetOperation._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            content = response.content.decode("utf-8")
            resp = operations_pb2.Operation()
            resp = json_format.Parse(content, resp)
            resp = self._interceptor.post_get_operation(resp)
            return resp

    @property
    def list_operations(self):
        return self._ListOperations(self._session, self._host, self._interceptor)  # type: ignore

    class _ListOperations(
        _BaseEngineServiceRestTransport._BaseListOperations, EngineServiceRestStub
    ):
        def __hash__(self):
            return hash("EngineServiceRestTransport.ListOperations")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: operations_pb2.ListOperationsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> operations_pb2.ListOperationsResponse:
            r"""Call the list operations method over HTTP.

            Args:
                request (operations_pb2.ListOperationsRequest):
                    The request object for ListOperations method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                operations_pb2.ListOperationsResponse: Response from ListOperations method.
            """

            http_options = (
                _BaseEngineServiceRestTransport._BaseListOperations._get_http_options()
            )
            request, metadata = self._interceptor.pre_list_operations(request, metadata)
            transcoded_request = _BaseEngineServiceRestTransport._BaseListOperations._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEngineServiceRestTransport._BaseListOperations._get_query_params_json(
                transcoded_request
            )

            # Send the request
            response = EngineServiceRestTransport._ListOperations._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            content = response.content.decode("utf-8")
            resp = operations_pb2.ListOperationsResponse()
            resp = json_format.Parse(content, resp)
            resp = self._interceptor.post_list_operations(resp)
            return resp

    @property
    def kind(self) -> str:
        return "rest"

    def close(self):
        self._session.close()


__all__ = ("EngineServiceRestTransport",)
